Class: Numeric

Inherits:
Object
  • Object
show all
Defined in:
lib/sys/filesystem.rb

Overview

Reopen the Numeric class and add some convenient methods for converting bytes to kb, mb, and gb.

Instance Method Summary collapse

Instance Method Details

#to_gbObject

call-seq:

<tt>num</tt>.to_gb

Returns num in terms of gigabytes.



143
144
145
# File 'lib/sys/filesystem.rb', line 143

def to_gb
  self / 1073741824
end

#to_kbObject

call-seq:

<tt>num</tt>.to_kb

Returns num in terms of kilobytes.



127
128
129
# File 'lib/sys/filesystem.rb', line 127

def to_kb
  self / 1024
end

#to_mbObject

call-seq:

<tt>num</tt>.to_mb

Returns num in terms of megabytes.



135
136
137
# File 'lib/sys/filesystem.rb', line 135

def to_mb
  self / 1048576
end

#to_tbObject

call-seq:

<tt>num</tt>.to_gb

Returns num in terms of terabytes.



151
152
153
# File 'lib/sys/filesystem.rb', line 151

def to_tb
  self / 1099511627776
end